##################################################################################
#
#   Kaivo Public Software License
#
#   Copyright (c) 2001, Kaivo, Inc.
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions are met:
#
#       o Redistributions of source code must retain the above copyright notice,
#         this list of conditions and the following disclaimer.
#
#       o Redistributions in binary form must reproduce the above copyright
#         notice, this list of conditions and the following disclaimer
#         in the documentation and/or other materials provided with the
#         distribution.
#
#       o Neither Kaivo nor the names of its contributors may be used to endorse
#         or promote products derived from this software without specific prior
#         written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#   ARE DISCLAIMED. IN NO EVENT SHALL KAIVO OR ITS CONTRIBUTORS BE LIABLE FOR 
#   ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
#   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
#   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND 
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
#   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##################################################################################

__doc__ = """Document Library Document Storage"""
__version__ = '0.2a'

# Document Store by Casey Duncan (cduncan@kaivo.com)
# Kaivo, Inc. (http://www.kaivo.com)

# This initial implementation uses a BTreeFolder to store the documents
# in the ZODB. Later implementations will abstract this to allow the
# option of storing the documents in a separate file system.

from string import strip, split, join, find
from StringIO import StringIO
import Globals
from Globals import DTMLFile
from App.ImageFile import ImageFile
from Products.BTreeFolder.BTreeFolder import BTreeFolder
from CatalogPlus import CatalogPlus
from DateTime.DateTime import DateTime
import DLGlobals
import Document 

class DocumentStore(BTreeFolder):
    """Document library document storage class"""

    meta_type="Document Store"
    all_meta_types=({'name': 'Document File', 'action': 'manage_addDocumentForm'},)

    #manage_options = BTreeFolder.manage_options + (
    #    {'label': 'Search MetaData', 'action': 'manage_catalogSchema'},
    #)

    __ac_permissions__ = BTreeFolder.__ac_permissions__ + (
        ('Add Document Library Files', ('manage_addDocumentForm', 'manage_addDocument',
                                        'addDocumentFile')),
        ('Delete objects', ('deleteDocumentFile',)),
        ('Search ZCatalog', ('query', 'getpath'))
    )

    _documentProperties = (
        { 'id': 'title', 'type': 'string', 'required': 1 },
        { 'id': 'content_type', 'type': 'string', 'hidden': 1 },
        { 'id': 'type', 'type': 'selection', 'select_variable': 'document_types', 'required': 1 },
        { 'id': 'description', 'type': 'text', 'default': '', 'required': 1  },
        { 'id': 'topics', 'type': 'lines', 'default': [], 'required': 1  },
        { 'id': 'creator', 'type': 'string', 'default': '' },
        { 'id': 'date', 'type': 'date', 'default': DLGlobals.document_no_creation_date },
        { 'id': 'review_date', 'type': 'date', 'default': DLGlobals.document_unreviewed_date },

    )

    # This will be externally manageable in the future
    _catalogColumns = ( 'id', 'title', 'date', 'description' )
    _catalogIndexes = (
        ('id', 'FieldIndex'),
        ('title', 'TextIndex'),
        ('type', 'FieldIndex'),
        ('description', 'TextIndex'),
        ('topics', 'KeywordIndex'),
        ('super_topics', 'KeywordIndex'),
        ('creator', 'TextIndex'),
        ('date', 'FieldIndex'),
        ('review_date', 'FieldIndex'),
        ('all_searchable_text', 'TextIndex')
    )

    def __init__(self, id, title='', vocab_id=None, container=None):
        self.id = id
        self.title = title

        if vocab_id is not None and container is None:
            raise AttributeError, 'You cannot specify a vocab_id without' \
                                  'also specifying a container.'
        if container is not None:
            self=self.__of__(container)
        
        # Setup the document store schema and indexes
        self.Catalog = CatalogPlus(vocab_id)
        
        for name in self._catalogColumns:
            self.Catalog.addColumn(name)

        for name, type in self._catalogIndexes:
            self.Catalog.addIndex(name, type)
        
        try: # Be flexible with the BTreeFolder implementation
            BTreeFolder.__init__(self)
        except:
            BTreeFolder.__init__(self, id)

    manage_reinit__roles__ = ('Manager',)
    def manage_reinit(self):
        """Debug code to reinit the catalog on schema changes"""

        self.Catalog = CatalogPlus(self.Catalog.lexicon)
       
        for name in self._catalogColumns:
            self.Catalog.addColumn(name)

        for name, type in self._catalogIndexes:
            self.Catalog.addIndex(name, type)

        for doc in self.objectValues():
            doc.setIcons()
            doc.index_object()

        return 'Operation succeeded'

    DocumentStore_icon = ImageFile('www/DocumentStore_icon.gif', globals())
    def icon(self):
        """Managment icon"""
        return join(self.getPhysicalPath()[1:],'/') + '/DocumentStore_icon'

    manage_addDocumentForm = Document.manage_addDocumentForm
    manage_addDocument = Document.manage_addDocument

    def defaultPropertyValue(self, propertyId, d=None):
        """Return the default value for a document property"""
        for p in self._documentProperties:
            if p['id'] == propertyId:
                try:
                    return p['default']
                except KeyError:
                    return d
        return d

    def documentPropertyErrors(self, REQUEST, error_template='<span class="form-error">%s</span>'):
        """Validates document properties passed on the REQUEST object
           returns an error string if errors are found and an empty string otherwise"""
        errors = {}
        for p in self._documentProperties:
            v = REQUEST.get(p['id'], None)

            if p.has_key('required') and p['required'] and not v:
                errors[p['id']] = error_template % 'Required Field'
            elif p['type'] == 'date' and v:
                try:
                    date = DateTime(v)
                    REQUEST.set(p['id'], v)
                except:
                    errors[p['id']] = error_template % 'Invalid Date'

        if errors:
            REQUEST.set('property_errors', errors)
            return error_template % "Errors were found on the form." \
                                    "please correct the errors and resubmit."
        else:
            return ''

    def addDocumentFile(self, REQUEST):
        """Adds a new document and sets its properties from a form based submission"""

        self=self.this()
        doc = Document.Document(title=REQUEST.title, container=self)
        self._setObject(doc.getId(), doc)
        doc = self._getOb(doc.getId())

        # Set the properties, but don't index yet
        doc.manage_changeProperties(REQUEST, reindex=0)
        # See if a file or url was specified
        if not REQUEST.file.filename and REQUEST.has_key('url') and REQUEST.url and REQUEST.url != 'http://':
            doc.manage_upload(file=REQUEST.url, remote=1)
        else:
            doc.manage_upload(REQUEST.file)

        if not Document.check_content_type(self, doc.content_type):
            self.manage_delObjects(doc.getId())
            raise "IllegalFileType", \
                  "<span>That file type cannot be uploaded into the library.</span>"

    def deleteDocumentFile(self, REQUEST=None, RESPONSE=None):
        """Deletes a document from the document store"""
        self.manage_delObjects(REQUEST.id)

        if REQUEST and RESPONSE and REQUEST.has_key('last_url'):
            RESPONSE.redirect(REQUEST.last_url + '?' + REQUEST.get('last_query',''))

    #
    #   Catalog query support methods
    #

    def query(self, REQUEST=None, **kw):
        """Query the document store for matching documents"""
        return apply(self.Catalog.searchResults, (REQUEST,), kw)

    def getpath(self, rid):
        """
        Return the path to a cataloged object given a 'data_record_id_'
        Used by the Catalog brains
        """
        return self.Catalog.paths[rid]

Globals.InitializeClass(DocumentStore)
